# Generated by Django 3.2.21 on 2024-02-12 13:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('payroll', '0017_csvreconciliationupload_historicalcsvreconciliationupload'),
    ]

    operations = [
        migrations.AlterField(
            model_name='benefitconsumption',
            name='receipt',
            field=models.CharField(blank=True, db_column='Receipt', max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='historicalbenefitconsumption',
            name='receipt',
            field=models.CharField(blank=True, db_column='Receipt', max_length=255, null=True),
        ),
    ]
