# Generated by Django 3.2.19 on 2023-08-11 07:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('payroll', '0006_auto_20230809_1057'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalpayroll',
            name='payment_method',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='payroll',
            name='payment_method',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
