# Generated by Django 4.2.20 on 2025-04-04 10:02

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("payroll", "0021_auto_20240715_0855"),
    ]

    operations = [
        migrations.AlterField(
            model_name="csvreconciliationupload",
            name="user_created",
            field=models.ForeignKey(
                db_column="UserCreatedUUID",
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="%(class)s_user_created",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="csvreconciliationupload",
            name="user_updated",
            field=models.ForeignKey(
                db_column="UserUpdatedUUID",
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="%(class)s_user_updated",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]
