# Generated by Django 3.2.21 on 2023-10-06 07:55

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('payroll', '0008_historicalpaymentadaptorhistory_paymentadaptorhistory'),
    ]

    operations = [
        migrations.AlterField(
            model_name='payroll',
            name='payment_point',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='payroll.paymentpoint'),
        ),
    ]
